# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "pypdf~=5.1.0",
#     "s3fs>=2024.10.0",
# ]
# [tool.uv]
# exclude-newer = "2024-11-18T00:00:00Z"
# ///

import argparse
from datetime import datetime

from pypdf import PdfReader


def read_pdf(file_path: str, page: int = 0):
    """
    Reads a PDF file from local or S3, and extracts text from specified page.

    Args:
        file_path (str): The path to the PDF file. Supports both local and S3 paths.
        page (int): If 0, extract text from all pages; otherwise, extract from the specified page index.

    Returns:
        dict: A dictionary with page numbers as keys and extracted text as values.
    """
    if file_path.startswith("s3://"):
        import s3fs  # type: ignore

        # Handle S3 files
        fs = s3fs.S3FileSystem()
        with fs.open(file_path, "rb") as f:  # Open the file in binary read mode
            reader = PdfReader(f)  # Pass the open file object directly to PdfReader
            number_of_pages = len(reader.pages)

            if page > 0:
                if page > number_of_pages:
                    raise ValueError(
                        f"Page index {page} is out of range (PDF has {number_of_pages} pages)."
                    )
                return {page: reader.pages[page - 1].extract_text()}
            else:
                return {
                    i + 1: reader.pages[i].extract_text()
                    for i in range(number_of_pages)
                }
    else:
        # Handle local files
        reader = PdfReader(file_path)
        number_of_pages = len(reader.pages)

        if page > 0:
            if page > number_of_pages:
                raise ValueError(
                    f"Page index {page} is out of range (PDF has {number_of_pages} pages)."
                )
            return {page: reader.pages[page - 1].extract_text()}
        else:
            return {
                i + 1: reader.pages[i].extract_text() for i in range(number_of_pages)
            }


def main(file_path: str, page: int = 0, debug: bool = False) -> str:
    """
    Extract text from a PDF file and print to stdout.

    Args:
        file_path (str): The path to the PDF file. Supports both local and S3 paths (requires s3fs).
        page (int): If 0, extract text from all pages; otherwise, extract from the specified page index.
        debug (bool): If True, skip printing the document

    Returns:
        str: Extracted text from the PDF file.
    """
    out = ""
    try:
        # Extract text
        text_data = read_pdf(file_path, page)
        if not debug:
            today = datetime.today().strftime("%Y-%m-%d:%H:%M:%S")
            out += f"File: {file_path}"
            out += f"Scraped: {today}"
        for page, text in text_data.items():
            if debug:
                out += f"{file_path}: Page {page:03d} OK"
            else:
                out += f"Page {page:03d}:\n{text}\n{'-' * 40}"
    except Exception as e:
        out += f"{file_path}:::Error: {e}"
    return out


def run():
    # Set up argument parser
    parser = argparse.ArgumentParser(
        description="Extract text from a PDF file (local or S3 if installed with `pdf-utils[s3]`)."
    )
    parser.add_argument(
        "file_path",
        type=str,
        help="The path to the PDF file. Supports local or S3 paths (e.g., 'example.pdf' or 's3://bucket-name/file.pdf').",
    )
    parser.add_argument(
        "--page",
        "-p",
        type=int,
        default=1,
        help="Page index to extract text from. Use 0 to extract all pages. Default is 1.",
    )
    parser.add_argument(
        "--debug",
        action="store_true",
        help="Skip printing document to stdout. Just print metadata and OK if successful.",
    )

    # Parse arguments
    args = parser.parse_args()
    out = main(file_path=args.file_path, page=args.page, debug=args.debug)
    print(out)


if __name__ == "__main__":
    run()
